require 'cgi'

class Board
  include Enumerable

  attr_reader :loaded


  # Initialize the board with the given number of rounds
  def initialize(num_rounds = 0, num_categories = 0, num_questions = 0)
    @rounds = (0...num_rounds).collect { Round.new(num_categories, num_questions) }
    #@rounds << Round.new(1, 1)  # Final round has only one category and question
    @loaded = 0
  end

  # Resize the board
  def resize(num_rounds)
    @rounds = (0...num_rounds).collect { Round.new(0, 0) }
    @loaded = 0
  end

  # Access the rounds by index
  def [](i)
    @rounds[i]
  end
  
  # Allow iteration through the rounds
  def each(*args, &block)
    @rounds.each(*args, &block)
  end
  
  # Return the size of the board
  def num_rounds
    @rounds.size
  end
  def num_categories_per_round
    self.num_rounds > 0 ? @rounds[0].num_categories : 0
  end
  def num_cells_per_category
    self.num_categories_per_round > 0 ? @rounds[0][0].num_cells : 0
  end
  def num_cells_total
    @rounds.inject(0) { |s, i| s + i.num_cells_total }
  end
  
  # Clear the 'done' flag in the entire board
  def clear
    @rounds.each { |round| round.clear }
  end
  
  # Access the last loaded round (for incomplete load)
  def last_round_loaded
    @rounds[@loaded]
  end

  # Access the last loaded round number (for incomplete load)
  def last_round_number_loaded
    @loaded + 1
  end

  # Load from a reader  
  def load(reader)
    begin
      args = reader.read_keyword('@BOARD', false, 1, 'Missing number of rounds after "@BOARD".')
      num_rounds = args[0].to_i
      raise IOError, 'Number of rounds must be positive.' if num_rounds <= 0
      resize(num_rounds)

      # Read in all the rounds
      @rounds.each { |round| round.load(reader); @loaded += 1 }
    rescue IOError => e
      raise IOError, "<p><b>Error at line #{reader.line_number}: #{e.message}</b><br />" +
                      "<div style='text-align:left;'><i>Lines in the vicinity:</i><br />" +
                      "<pre>#{CGI::escapeHTML(reader.lines_around_current)}</pre></div></p>"
    rescue EOFError => e
      raise IOError, "<p><b>Premature end of file.</b> Stopped in " +
            ( (@loaded == @categories.size) ?
              "final category" :
              ("round #{last_round_number_loaded}, " +
               "category #{last_round_loaded.last_category_number_loaded}, " +
               "question #{last_round_loaded.last_category_loaded.last_cell_number_loaded}") ) +
            "</p>"
    end

    true
  end
end