require 'formatted_text'

class Cell
  attr_accessor :daily_double, :wild, :done, :last_choice
  attr_reader   :question, :answer, :ans

  DEFAULT_QUESTION_SIZE = '1.5em'
  DEFAULT_ANSWER_SIZE = '2em'
  CHOICE_CORRECT = ?A
  CHOICE_WRONG   = ?B
  TEXT_CORRECT = 'Correct'
  TEXT_WRONG   = 'Incorrect'
  IMG_CORRECT = 'correct.png'
  IMG_WRONG   = 'wrong.png'
  #DEFAULT_LAST_CHOICE = ?E  # Choices allowed are A through the character in @last_choice

  def initialize
    @question = FormattedText.new
    @answer   = FormattedText.new
    @ans      = nil
    @daily_double = false
    @wild = false
    @done = false
    @last_choice = nil #= DEFAULT_LAST_CHOICE
  end
  
  # Clear the cell's 'done' flag
  def clear
    @done = false
  end
  
  def question=(q)
    @question.text = q
  end

  def answer=(a)
    @answer.text = a
  end

  def choices(game_controller, text_only=false)
    choice_list = self.last_choice ?
      (?A..self.last_choice).collect { |c| '%c' % c }  <<  '-' :   # Add '-' as the final choice always
      [Cell::CHOICE_CORRECT, Cell::CHOICE_WRONG].collect { |c| '%c' % c }
    text_list = self.last_choice ? choice_list :
      text_only ? [ Cell::TEXT_CORRECT, Cell::TEXT_WRONG ] :
        [ game_controller.image(Cell::IMG_CORRECT, :alt => Cell::TEXT_CORRECT, :title => Cell::TEXT_CORRECT, :style => 'width:2em;height:2em;'),
          game_controller.image(Cell::IMG_WRONG  , :alt => Cell::TEXT_WRONG  , :title => Cell::TEXT_WRONG  , :style => 'width:2em;height:2em;') ]

    text_list.zip(choice_list)
  end
  
  def correct_answer?(ans)
    (@ans and ans == @ans) or (!@ans and ans == Cell::CHOICE_CORRECT)
  end

  def multiple_choice?
    self.last_choice != nil
  end

  def binary?
    self.last_choice == ?B
  end
  
  
  def max_num_guesses
      if daily_double or binary?
        return 1
      elsif !multiple_choice?
        return GameController::NUM_PLAYERS
      else # for multiple choices with more than 2 choices
        return GameController::MAX_NUM_GUESSES
      end
  end

  # Load from a reader
  def load(reader)
    # Read in the header for the question
    @question.style_size, args = reader.read_keyword('@QUESTION', true, [0, 1], nil)
    @question.style_size = DEFAULT_QUESTION_SIZE if !@question.style_size
    dd = args[0]
    if dd and dd.size > 0 and dd != 'DD' and dd != 'WC'
        raise IOError, "Unexpected \"#{dd}\" after @QUESTION. Did you mean DD (for Daily Double) or WC (for wildcard)"
    end
    self.daily_double = (dd == 'DD')
    self.wild = (dd == 'WC')

    # Read in the question
    @question.text = reader.readlines_til(/^\@/, true).rstrip
    @last_choice = Cell.guess_last_choice(@question.text)
    @question.color_options_til = @last_choice

    # Read in the header for the answer
    @answer.style_size, args = reader.read_keyword('@ANSWER', true, 0, nil)
    @answer.style_size = DEFAULT_ANSWER_SIZE if !@answer.style_size

    # Read in the answer
    @answer.text = reader.readlines_til(/^\@/, true).rstrip
    @answer.color_options_til = @last_choice

    if (@last_choice)
      # Choose the first letter as the answer and ensure that the answer lies
      # within the choices allowed.
      raise IOError, "The answer (#{@answer}) seems to be missing." if @answer.text !~ /^\s*(\S)/mi
      @ans = $1[0]  # Store it as a character

      if @ans < ?A or @ans > @last_choice
        raise IOError, "First non-space character of answer should be '#{'A-' + ('%c' % @last_choice)}'." +
                        " Received '#{'%c' % @ans}'."
      end
    else
      @ans = nil
    end
  end
  
  # Guess the last choice by parsing the question text.
  # OBSOLETE: By default, assume DEFAULT_LAST_CHOICE.
  def Cell.guess_last_choice(text)
    last_choice = nil

    # Figure out the last choice by parsing
    for choice in ?A..?Z
      if text =~ /(?:\s|(?:\&nbsp;))+#{'%c' % choice}\./
        last_choice = choice
      else
        break
      end
    end

    last_choice # or DEFAULT_LAST_CHOICE
  end
end
