class Round
  include Enumerable

  attr_reader :loaded
  attr_reader :intro

  # Initialize the board with the given number of categories
  def initialize(num_categories = 0, num_cells = 0)
    @categories = (0...num_categories).collect { Category.new(num_cells) }
    @loaded = 0
    @intro = FormattedText.new
  end

  # Resize the round
  def resize(num_categories, num_cells)
    @categories = (0...num_categories).collect { Category.new(num_cells) }
    @loaded = 0
  end
  
  # Access the categories by index
  def [](i)
    @categories[i]
  end

  # Return the size of the board
  def num_categories
    @categories.size
  end
  def num_cells_per_category
    @categories[0].num_cells
  end
  def num_cells_total
    @categories.inject(0) { |s, i| s + i.num_cells }
  end
  
  # Allow iteration through the categories
  def each(*args, &block)
    @categories.each(*args, &block)
  end

  # Clear the 'done' flag in the entire board
  def clear
    @categories.each { |category| category.clear }
  end
  
  # Access the last loaded category (for incomplete load)
  def last_category_loaded
    @categories[@loaded]
  end

  # Access the last loaded category number (for incomplete load)
  def last_category_number_loaded
    @loaded + 1
  end
  
  # Load from a reader  
  def load(reader)
    # Read in the header for the round
    @intro.style_size, args = reader.read_keyword('@ROUND', true, 3, 'Missing number of categories and/or number of questions after "@ROUND".')

    # Initialize the number of categories
    num_cats, num_cells = args[0].to_i, args[1].to_i
    raise IOError, 'Number of categories and number of questions must be positive.' if num_cats <= 0 or num_cells <= 0
    resize(num_cats, num_cells)

    # Read in the intro
    @intro.text = reader.readlines_til(/^\@/, true).rstrip

    # Read in all the categories
    @categories.each do |category|
      category.resize(num_cells)
      category.load(reader) #, (num_cats != 1 or num_cells != 1))
      @loaded += 1
    end
  end
end